<?php

namespace AustraliaPost\Extensions\Business\Endpoints;

use AustraliaPost\Extensions\Business\API\Wrapper\Address;
use AustraliaPost\Extensions\Business\API\Wrapper\Price_Feature;
use AustraliaPost\Extensions\Business\API\Wrapper\Price_Item;
use AustraliaPost\Extensions\Business\Endpoints\Errors\Errors_Factory;
use AustraliaPost\Extensions\Business\Endpoints\Errors\Generic_Error;
use AustraliaPost\Extensions\Business\Helpers\Utilities as LabelsProUtilities;
use AustraliaPost\Helpers\Utilities;
use WC_Order;

abstract class Abstract_Endpoint {

	public function __construct() {
		add_action('wp_ajax_' . $this->action(), [$this, 'process']);
	}

	public function process() {

		check_ajax_referer( $this->action(), '_ajax_nonce' );

		$data = json_decode(file_get_contents('php://input'), true);

		$response = $this->callback($data);

		echo json_encode($response);
		exit;
	}

	/**
	 * @param array|string $errors
	 */
	protected function abort($errors)
	{
		$errorsMessages = [];

		if (is_array($errors)) {
			foreach($errors as $error) {
				if (isset($error['code']) || isset($error['error_code'])) {
					$errorsMessages[] = Errors_Factory::make($error)->error_message();
				}
			}
		} else {
			$errorsMessages = [$errors];
		}

		if ($errors === []) {
			$errorsMessages[] = (new Generic_Error([]))->error_message();
		}

		echo json_encode(['errors' => $errorsMessages]);
		exit;
	}

	protected function get_order($data) {
		$order_id = intval( $data['order_id'] );

		try {
			$order = new \WC_Order($order_id);
		}catch (\Exception $e) {
			$this->abort($e->getMessage());
		}

		return $order;
	}

	/**
	 * @return Address
	 */
	protected function get_from_address()
	{
		$address = new Address();

		$address->setName(apply_filters('labels_pro_from_address_name', get_bloginfo('name')));
		$address->addLine(WC()->countries->get_base_address());
		$address->addLine(WC()->countries->get_base_address_2());
		$address->setPostcode(WC()->countries->get_base_postcode());
		$address->setSuburb(WC()->countries->get_base_city());
		$address->setState(WC()->countries->get_base_state());
		$address->setCountry('AU');
		return $address;
	}

	/**
	 * @param WC_Order $order
	 *
	 * @return Address
	 */
	protected function get_to_address($order)
	{
		$address = new Address();

		$address->setName($order->get_shipping_first_name(). ' ' . $order->get_shipping_last_name());

		if (strlen(trim($order->get_shipping_company())) > 0) {
			$address->addLine($order->get_shipping_company());
		}

		$address->addLine($order->get_shipping_address_1());
		$address->addLine($order->get_shipping_address_2());
		$address->setPostcode($order->get_shipping_postcode());
		$address->setSuburb($order->get_shipping_city());
		$address->setState($order->get_shipping_state());
		$address->setCountry($order->get_shipping_country());
		$address->setEmail($order->get_billing_email());
		$address->setPhone($order->get_billing_phone());

		$shouldAddDeliveryInstruction = Utilities::get_method_setting('add_instructions_to_labels');

		if ($shouldAddDeliveryInstruction === 'yes') {
			$address->setDeliveryInstructions($order->get_customer_note());
		}

		return $address;
	}

	/**
	 * @param Price_Item $priceItem
	 * @param array $shipment
	 *
	 * @return Price_Item
	 */
	protected function addProductIds( $priceItem, $shipment )
	{
		if ( LabelsProUtilities::isMyPost()) {
			$priceItem->setProductIds( [ $shipment['product_id'] ] );
			return $priceItem;
		}

		$priceItem->setProductId($shipment['product_id']);
		return $priceItem;
	}

	/**
	 * @param Price_Item $priceItem
	 * @param array $shipment
	 * @param bool $force
	 * @return Price_Item
	 */
	protected function addSignatureOnDelivery( $priceItem, $shipment, $force = false )
	{
		if (!$shipment['signature_on_delivery'] && $force === false) {
			return $priceItem;
		}

		if (LabelsProUtilities::isMyPost()) {
			$sod_feature = new Price_Feature();
			$sod_feature->setCode('SIGNATURE_ON_DELIVERY');
			$sod_feature->setAttributes([]);
			$priceItem->addFeature($sod_feature);
			return $priceItem;
		}

		$priceItem->setSignatureOnDelivery(true);
		return $priceItem;
	}

	/**
	 * @param array $shipment
	 *
	 * @return int
	 */
	protected function cubicWeight($shipment)
	{
		// cubicMeter = (length * width * height) * 250 (dimensions in meters)
		$cubicWeightInKg = (($shipment['length'] * 0.001) * ($shipment['width'] * 0.001) * ($shipment['height'] * 0.001)) * 250;
		// convert to g and return either actual weight or cubic weight whichever greater.
		return $cubicWeightInKg * 1000;
	}

	/**
	 * @param mixed
	 * @return array
	 */
	public abstract function callback($data);

	/**
	 * @return string
	 */
	public abstract function action();

}
